import 'dart:convert';
import 'dart:io';
import 'package:flutter/material.dart';
import 'package:geolocator/geolocator.dart';
import 'package:intl/intl.dart';
import 'package:uuid/uuid.dart';
import 'package:dio/dio.dart';
import 'package:http/http.dart' as http;

Type typeOfElementsInList<T>(List<T> e) => T;

class HexColor1 extends Color {
  HexColor1(final String hexColor) : super(_getColorFromHex(hexColor));

  static int _getColorFromHex(String hexColor) {
    hexColor = hexColor.toUpperCase().replaceAll('#', '');
    if (hexColor.length == 6) {
      hexColor = 'FF' + hexColor;
    }
    return int.parse(hexColor, radix: 16);
  }
}

class UtilRF {
  static TextTheme _buildTextTheme(TextTheme base) {
    const String fontName = 'WorkSans';
    return base.copyWith(
      headline1: base.headline1.copyWith(fontFamily: fontName),
      headline2: base.headline2.copyWith(fontFamily: fontName),
      headline3: base.headline3.copyWith(fontFamily: fontName),
      headline4: base.headline4.copyWith(fontFamily: fontName),
      headline5: base.headline5.copyWith(fontFamily: fontName),
      headline6: base.headline6.copyWith(fontFamily: fontName),
      button: base.button.copyWith(fontFamily: fontName),
      caption: base.caption.copyWith(fontFamily: fontName),
      bodyText1: base.bodyText1.copyWith(fontFamily: fontName),
      bodyText2: base.bodyText2.copyWith(fontFamily: fontName),
      subtitle1: base.subtitle1.copyWith(fontFamily: fontName),
      subtitle2: base.subtitle2.copyWith(fontFamily: fontName),
      overline: base.overline.copyWith(fontFamily: fontName),
    );
  }

  static ThemeData buildLightTheme() {
    final Color primaryColor = HexColor1('#54D3C2');
    final Color secondaryColor = HexColor1('#54D3C2');
    final ColorScheme colorScheme = const ColorScheme.light().copyWith(
      primary: primaryColor,
      secondary: secondaryColor,
    );
    final ThemeData base = ThemeData.light();
    return base.copyWith(
      colorScheme: colorScheme,
      primaryColor: primaryColor,
      buttonColor: primaryColor,
      indicatorColor: Colors.white,
      splashColor: Colors.white24,
      splashFactory: InkRipple.splashFactory,
      accentColor: secondaryColor,
      canvasColor: Colors.white,
      backgroundColor: const Color(0xFFFFFFFF),
      scaffoldBackgroundColor: const Color(0xFFF6F6F6),
      errorColor: const Color(0xFFB00020),
      buttonTheme: ButtonThemeData(
        colorScheme: colorScheme,
        textTheme: ButtonTextTheme.primary,
      ),
      textTheme: _buildTextTheme(base.textTheme),
      primaryTextTheme: _buildTextTheme(base.primaryTextTheme),
      accentTextTheme: _buildTextTheme(base.accentTextTheme),
      platform: TargetPlatform.iOS,
    );
  }

  static libImage(String path, String libName) {
    return Image.asset(AssetImage('${path}').assetName, package: '${libName}');
  }

  static String getDateTime() {
    DateFormat dateFormat = DateFormat("yyyy-MM-dd HH:mm:ss");
    String currentDateTime = dateFormat.format(DateTime.now());
    return currentDateTime;
  }

  static String getDate() {
    DateFormat dateFormat = DateFormat("yyyy-MM-dd");
    String currentDateTime = dateFormat.format(DateTime.now());
    return currentDateTime;
  }

  static String getTime() {
    DateFormat dateFormat = DateFormat("HH:mm:ss");
    String currentDateTime = dateFormat.format(DateTime.now());
    return currentDateTime;
  }

  static geoPosition() async {
    Position pos;
    pos = await Geolocator.getCurrentPosition();
    return pos;
  }

  static getUUID() {
    return Uuid().v1();
  }

  static Future downloadFile(String url, String storePath, Function onStart,
      Function onProgress, Function onFinish, Function onError) async {
    try {
      onStart();
      var dio = Dio();
      Response response = await dio.download(
        url,
        storePath,
        onReceiveProgress: (int count, int total) {
          if (total != -1) {
            double downloaded = (count / total * 100);
            onProgress(downloaded);
            print(downloaded.toString() + "%");
            if (downloaded.toString() == '100.0') {
              onFinish();
            }
          }
        },
        //Received data with List<int>
        options: Options(
            responseType: ResponseType.bytes,
            followRedirects: false,
            validateStatus: (status) {
              return status < 500;
            }),
      );
      print(response.headers);
    } catch (e) {
      print('Error:' + e.toString());
      onError();
    }
  }

  static Future<http.StreamedResponse> uploadFile(
      {String url, File file, Function onComplete, Function onError}) async {
    var request = http.MultipartRequest('POST', Uri.parse(url));
    request.files.add(await http.MultipartFile.fromPath('File', file.path));

    http.StreamedResponse response = await request.send();

    if (response.statusCode == 200) {
      String resposeString = await response.stream.bytesToString();
      print(resposeString);
      onComplete(resposeString);
    } else {
      print(response.reasonPhrase);
      onError(response.reasonPhrase);
    }
    return response;
  }
}
