import 'package:flutter/material.dart';

//region ButtonRF
enum ButtonType { raised, outline, icon, flat, image }
enum ButtonStyleRF { info, danger, warning, submit, cancel, success }

class ButtonRF extends StatefulWidget {
  final ButtonType buttonType;
  final ButtonStyleRF buttonStyle;
  final Text text;
  final Icon icon;
  final Image image;
  final Color color;
  final Color textColor;
  final Color splashColor;
  final double elevation;
  final double width;
  final double height;
  final double borderRadious;
  final Function onTap;

  @override
  _ButtonRFState createState() => _ButtonRFState();

  ButtonRF({
    this.buttonType = ButtonType.raised,
    this.buttonStyle,
    this.text,
    this.icon,
    this.image,
    this.color,
    this.textColor = Colors.black87,
    this.splashColor,
    this.elevation,
    this.width = 100,
    this.height = 45,
    this.borderRadious = 0,
    this.onTap,
  });
}

class _ButtonRFState extends State<ButtonRF> {
  @override
  Widget build(BuildContext context) {
    Color buttonColor = widget.color;
    if (buttonColor == null) {
      switch (widget.buttonStyle) {
        case ButtonStyleRF.info:
          buttonColor = Colors.white;
          break;
        case ButtonStyleRF.danger:
          buttonColor = Colors.red;
          break;
        case ButtonStyleRF.warning:
          buttonColor = Colors.orange;
          break;
        case ButtonStyleRF.submit:
          buttonColor = Colors.blue;
          break;
        case ButtonStyleRF.cancel:
          buttonColor = Colors.red;
          break;
        case ButtonStyleRF.success:
          buttonColor = Colors.green;
          break;
      }
    }

    switch (widget.buttonType) {
      case ButtonType.raised:
        return ButtonTheme(
          minWidth: widget.width,
          height: widget.height,
          child: RaisedButton(
            child: Row(
              mainAxisSize: MainAxisSize.min,
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[widget.text],
            ),
            onPressed: widget.onTap,
            color: buttonColor,
            elevation: widget.elevation,
            textColor: widget.textColor,
            splashColor: widget.splashColor,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(widget.borderRadious),
            ),
          ),
        );
        break;

      case ButtonType.outline:
        return ButtonTheme(
          minWidth: widget.width,
          height: widget.height,
          child: OutlineButton(
            child: Row(
              mainAxisSize: MainAxisSize.min,
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[widget.text],
            ),
            onPressed: widget.onTap,
            color: widget.color,
            textColor: widget.textColor,
            splashColor: widget.splashColor,
            borderSide: BorderSide(color: buttonColor),
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(widget.borderRadious),
            ),
          ),
        );
        break;
      case ButtonType.icon:
        return IconButton(
          icon: new Icon(
            Icons.search,
            color: buttonColor,
          ),
          onPressed: widget.onTap,
          color: buttonColor,
          splashColor: widget.splashColor,
        );
        break;
      case ButtonType.flat:
        return ButtonTheme(
          minWidth: widget.width,
          height: widget.height,
          child: FlatButton(
            child: Row(
              mainAxisSize: MainAxisSize.min,
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[widget.text],
            ),
            onPressed: widget.onTap,
            color: buttonColor,
            textColor: widget.textColor,
            splashColor: widget.splashColor,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(widget.borderRadious),
            ),
          ),
        );
        break;
      case ButtonType.image:
        return ButtonTheme(
          minWidth: widget.width,
          height: widget.height,
          child: RaisedButton(
            child: Row(
                mainAxisSize: MainAxisSize.min,
                mainAxisAlignment: MainAxisAlignment.center,
                children: <Widget>[
                  Expanded(flex: 1, child: widget.image),
                  Expanded(
                      flex: 2, child: Padding(padding: const EdgeInsets.only(left: 10), child: widget.text))
                ]),
            onPressed: widget.onTap,
            color: buttonColor,
            textColor: widget.textColor,
            splashColor: widget.splashColor,
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(widget.borderRadious),
            ),
          ),
        );
        break;
    }
    return Container();
  }
}
//endregion
